
# -*- encoding:euc-jp -*-

# λάǤǤѡ롣
# Web֥饦¸ȼȤȤΤǡDTDǧΤ̤ΤǤϤʤ
# ǧʤΤƤ褦ˤ롣

require "test/unit"
require "test/unit/ui/console/testrunner"

require "amrita/template"

class AmParseTest < Test::Unit::TestCase
  include Amrita
  
  def test10
    r = HtmlParser.parse_text '<html>foo</html>'
    assert_instance_of(Element, r)   # 롼ǤҤȤĤΤȤElement
    assert_equal(e(:html){"foo"}, r)
  end
  
  def test10b
    r = HtmlParser.parse_text '<h1 id=r>head</h1><p id=s>text</p>'  # test-amrita/test10
    assert_instance_of(NodeArray, r)  # 롼ǤʣΤȤNodeArray
    assert_equal([e(:h1, :id=>"r"){"head"}, e(:p, :id=>"s"){"text"}], r.to_a)
  end
  
  def test10c
    parser = HtmlParser.new(<<EOF, nil, nil, DefaultHtmlTagInfo)  # test-amrita/test70
<?xml version="1.0"?>
<foo xmlns:t="amrita"><p t:tmplid="foo">should be replaced</p></foo>
EOF
    r = parser.parse()
    assert_instance_of(NodeArray, r)
    assert_instance_of(SpecialElement, r[0])
    assert_equal(special_tag("?", 'xml version="1.0"'), r[0])
    assert_instance_of(String, r[1].to_s)
    assert_equal("\n", r[1].to_s)
    assert_equal(e(:foo, "xmlns:t".intern=>"amrita") {
      e(:p, "t:tmplid".intern=>"foo"){"should be replaced"}}, r[2])
    assert_equal("\n", r[3].to_s)
  end

  def test11
    r = HtmlParser.parse_text '<textarea />'
    assert_equal(e(:textarea){}, r)
  end

  # to_a
  def test15
    r = HtmlParser.parse_text '<p>foo<i></i><p>bar'
    ary = [e(:p) {["foo", e(:i){}]}, e(:p) {"bar"}]
    assert_equal(NodeArray.new(*ary), r)
    assert_equal(ary, r.to_a)
  end

  # λάǤ
  # <!ELEMENT P - O (%inline;)*            -- paragraph -->
  def test21
    r = HtmlParser.parse_text '<p>foo<i>bar</i>baz<ul></ul>hoge'
    assert_equal([e(:p) {["foo", e(:i) {"bar"}, "baz"]}, e(:ul) {}, text("hoge")],
                 r.to_a)

    r = HtmlParser.parse_text '<p>hoge<p>hoge'
    assert_equal([e(:p) {"hoge"}, e(:p) {"hoge"}], r.to_a)
  end

  # <!ELEMENT DT - O (%inline;)*           -- definition term -->
  # <!ELEMENT DD - O (%flow;)*             -- definition description -->
  def test22
    r = HtmlParser.parse_text '<dt>foo<dd>ba<ul></ul>r<dt>baz'
    assert_equal([e(:dt) {"foo"}, e(:dd) {["ba", e(:ul) {}, "r"]}, e(:dt) {"baz"}], r.to_a)

    r = HtmlParser.parse_text '<dt>hoge<dt>hoge'
    assert_equal([e(:dt) {"hoge"}, e(:dt) {"hoge"}], r.to_a)
    
    r = HtmlParser.parse_text '<dd>a<dd>b'
    assert_equal([e(:dd) {"a"}, e(:dd) {"b"}], r.to_a)
  end

  # <!ELEMENT LI - O (%flow;)*             -- list item -->
  def test23
    r = HtmlParser.parse_text '<li>a<ul></ul>b<li>c'
    assert_equal([e(:li) {["a", e(:ul) {}, "b"]}, e(:li) {"c"}], r.to_a)
  end

  # <!ELEMENT SELECT - - (OPTGROUP|OPTION)+ -- option selector -->
  # <!ELEMENT OPTGROUP - - (OPTION)+ -- option group -->
  # <!ELEMENT OPTION - O (#PCDATA)         -- selectable choice -->
  def test24
    r = HtmlParser.parse_text '<option>a<option>b'
    assert_equal([e(:option) {"a"}, e(:option) {"b"}], r.to_a)

    r = HtmlParser.parse_text '<option>a<optgroup><option>g1</optgroup>'
    assert_equal([e(:option) {"a"}, e(:optgroup) {e(:option) {"g1"}}], r.to_a)
  end

  # <!ELEMENT TABLE - - (CAPTION?, (COL*|COLGROUP*), THEAD?, TFOOT?, TBODY+)>
  # <!ELEMENT CAPTION  - - (%inline;)*     -- table caption -->
  # <!ELEMENT THEAD    - O (TR)+           -- table header -->
  # <!ELEMENT TFOOT    - O (TR)+           -- table footer -->
  # <!ELEMENT TBODY    O O (TR)+           -- table body -->
  # <!ELEMENT COLGROUP - O (COL)*          -- table column group -->
  # <!ELEMENT COL      - O EMPTY           -- table column -->
  # <!ELEMENT TR       - O (TH|TD)+        -- table row -->
  # <!ELEMENT (TH|TD)  - O (%flow;)*       -- table header cell, table data cell-->
  def test25
    r = HtmlParser.parse_text(
                 '<thead><tr><td>header<tfoot><tr><td>footer<tbody><tr><td>row')
    assert_equal(
      [e(:thead) {e(:tr) {e(:td) {"header"}}},
           e(:tfoot) {e(:tr) {e(:td) {"footer"}}}, e(:tbody) {e(:tr) {e(:td) {"row"}}}],
      r.to_a)

    r = HtmlParser.parse_text '<tbody><tr><td>first<tbody><tr><td>second'
    assert_equal(
      [e(:tbody) {e(:tr) {e(:td) {"first"}}}, e(:tbody) {e(:tr) {e(:td) {"second"}}}],
      r.to_a)

    r = HtmlParser.parse_text(
               '<colgroup><col width="20"><col width="20"><tbody><tr><td>row')
    assert_equal(
      [e(:colgroup) {[e(:col, :width=>"20"), e(:col, :width=>"20")]},
      e(:tbody) {e(:tr) {e(:td) {"row"}}}],
      r.to_a)

    r = HtmlParser.parse_text '<tr><td>1<th>2<tr><th>3<td>4<tr><td>5'
    assert_equal(
      [e(:tr){[e(:td){"1"}, e(:th){"2"}]}, e(:tr){[e(:th){"3"}, e(:td){"4"}]},
       e(:tr){e(:td){"5"}}],
      r.to_a)
  end

  def test31
    r = HtmlParser.parse_text('<foo id="a" s="@b">hoge</foo>')
    assert_equal([e(:foo, :id=>"a", :s=>"@b") {"hoge"}], r.to_a)
  end
end

if __FILE__ == $0
  Test::Unit::UI::Console::TestRunner.run(AmParseTest)
end
